"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
/* eslint-disable no-await-in-loop */
/* eslint-disable @typescript-eslint/no-var-requires */
/* eslint-disable global-require */
/* eslint-disable import/no-dynamic-require */
const core_1 = require("@oclif/core");
const globby_1 = __importDefault(require("globby"));
const path_1 = __importDefault(require("path"));
const fs_1 = require("fs");
const mkdirp_1 = __importDefault(require("mkdirp"));
const log_symbols_1 = __importDefault(require("log-symbols"));
const figures_1 = __importDefault(require("figures"));
const chalk_1 = __importDefault(require("chalk"));
const base_command_1 = require("../../utils/base-command");
const fs_2 = require("../../utils/fs");
const babel_register_1 = require("../../utils/babel-register");
// import Validate from '../json/validate';
/** A command to compile player DSL content into JSON */
class DSLCompile extends base_command_1.BaseCommand {
    getOptions() {
        var _a, _b, _c, _d, _e, _f, _g, _h;
        return __awaiter(this, void 0, void 0, function* () {
            const { flags } = yield this.parse(DSLCompile);
            const config = yield this.getPlayerConfig();
            const input = (_a = flags.input) !== null && _a !== void 0 ? _a : (_b = config.dsl) === null || _b === void 0 ? void 0 : _b.src;
            if (!input) {
                throw new Error(`Input files are required for DSL compilation`);
            }
            return {
                input,
                output: (_e = (_c = flags.output) !== null && _c !== void 0 ? _c : (_d = config.dsl) === null || _d === void 0 ? void 0 : _d.outDir) !== null && _e !== void 0 ? _e : '_out',
                skipValidation: (_h = (_f = flags['skip-validation']) !== null && _f !== void 0 ? _f : (_g = config.dsl) === null || _g === void 0 ? void 0 : _g.skipValidation) !== null && _h !== void 0 ? _h : false,
            };
        });
    }
    run() {
        return __awaiter(this, void 0, void 0, function* () {
            const { input, output, skipValidation } = yield this.getOptions();
            const files = yield (0, globby_1.default)((0, fs_2.convertToFileGlob)([input], '**/*.(tsx|jsx|js|ts)'), {
                expandDirectories: true,
            });
            (0, babel_register_1.registerForPaths)();
            this.debug('Found %i files to process', files.length);
            const results = {
                exitCode: 0,
            };
            /** Compile a file from the DSL format into JSON */
            const compileFile = (file) => __awaiter(this, void 0, void 0, function* () {
                const compiler = yield this.createDSLCompiler();
                const requiredModule = require(path_1.default.resolve(file));
                const defaultExport = requiredModule.default;
                if (!defaultExport) {
                    return;
                }
                let relativePath = path_1.default.relative(input, file);
                if (!relativePath) {
                    relativePath = path_1.default.basename(file);
                }
                const outputFile = path_1.default.join(output, path_1.default.format(Object.assign(Object.assign({}, path_1.default.parse(relativePath)), { base: undefined, ext: '.json' })));
                this.log(`${log_symbols_1.default.info} Compiling %s ${figures_1.default.arrowRight} %s`, (0, fs_2.normalizePath)(file), (0, fs_2.normalizePath)(outputFile));
                const { value, contentType, sourceMap } = yield compiler.serialize(defaultExport);
                const contentStr = JSON.stringify(value, null, 2);
                yield (0, mkdirp_1.default)(path_1.default.dirname(outputFile));
                yield fs_1.promises.writeFile(outputFile, contentStr);
                if (sourceMap) {
                    yield fs_1.promises.writeFile(`${outputFile}.map`, sourceMap);
                }
                return {
                    contentType,
                    outputFile,
                    inputFile: file,
                };
            });
            const compilerResults = [];
            // This has to be done serially b/c of the way React logs messages to console.error
            // Otherwise the errors in console will be randomly interspersed between update messages
            for (let fIndex = 0; fIndex < files.length; fIndex += 1) {
                const file = files[fIndex];
                try {
                    const result = yield compileFile(file);
                    compilerResults.push({
                        output: result,
                        state: 'completed',
                    });
                }
                catch (e) {
                    results.exitCode = 100;
                    console.log('');
                    console.log(chalk_1.default.red(`${log_symbols_1.default.error} Error compiling ${file}: ${e.message}`));
                    this.debug(e);
                    compilerResults.push({
                        state: 'completed',
                        error: e,
                    });
                }
            }
            // if (!skipValidation) {
            //   console.log('');
            //   const hasOutput = compilerResults.some(
            //     (r) => r.output?.contentType === 'flow'
            //   );
            //   if (hasOutput) {
            //     await Validate.run([
            //       '-f',
            //       ...compilerResults
            //         .filter((r) => r.output?.contentType === 'flow')
            //         .map((result) => {
            //           return result.output?.outputFile ?? '';
            //         }),
            //     ]);
            //   } else {
            //     console.log('No output to validate');
            //   }
            // }
            this.exit(results.exitCode);
            return results;
        });
    }
}
exports.default = DSLCompile;
DSLCompile.description = 'Compile Player DSL files into JSON';
DSLCompile.flags = Object.assign(Object.assign({}, base_command_1.BaseCommand.flags), { input: core_1.Flags.string({
        char: 'i',
        description: 'An input directory to compile.\nAny jsx/ts/tsx files will be loaded via babel-require automatically.',
    }), output: core_1.Flags.string({
        char: 'o',
        description: 'Output directory to write results to.',
    }), 'skip-validation': core_1.Flags.boolean({
        description: 'Option to skip validating the generated JSON',
    }) });
