"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.createTaskRunner = void 0;
const log_update_1 = __importDefault(require("log-update"));
/** Create a runner to kick off tasks in parallel */
const createTaskRunner = ({ tasks, renderer, }) => {
    const statefulTasks = tasks.map((t) => {
        return Object.assign(Object.assign({}, t), { state: 'idle' });
    });
    /** Kick off the task list */
    const run = () => __awaiter(void 0, void 0, void 0, function* () {
        const startTime = Date.now();
        let ended = false;
        const paintInterval = setInterval(() => {
            if (ended) {
                return;
            }
            const output = renderer.onUpdate({ tasks: statefulTasks });
            if (process.stdout.isTTY) {
                (0, log_update_1.default)(output);
            }
        }, 10);
        yield Promise.all(statefulTasks.map((t) => __awaiter(void 0, void 0, void 0, function* () {
            t.state = 'pending';
            try {
                const r = yield t.run();
                t.state = 'completed';
                t.output = r;
            }
            catch (e) {
                if (e instanceof Error) {
                    t.state = 'completed';
                    t.error = e;
                }
            }
        })));
        ended = true;
        clearInterval(paintInterval);
        const duration = Date.now() - startTime;
        const output = renderer.onEnd({
            duration,
            tasks: statefulTasks,
        });
        console.log(output);
        log_update_1.default.done();
        return statefulTasks;
    });
    return {
        tasks: statefulTasks,
        run,
    };
};
exports.createTaskRunner = createTaskRunner;
